﻿namespace Web.Binding {
    using System;
    using System.CodeDom;
    using System.Collections.Generic;
    using System.Linq;

    internal static class CodeDomUtility {
        internal static string GetControlType(CodeMemberMethod buildMethod) {
            var controlDeclarationStatement = buildMethod.Statements[0] as CodeVariableDeclarationStatement;
            if (controlDeclarationStatement == null) {
                return String.Empty;
            }
            return controlDeclarationStatement.Type.BaseType;
        }

        internal static string GetControlID(CodeMemberMethod buildMethod) {
            // We only support string constants for control ids
            return (from s in buildMethod.Statements.OfType<CodeAssignStatement>()
                    let left = s.Left as CodePropertyReferenceExpression                    
                    let right = s.Right as CodePrimitiveExpression
                    where left != null && right != null && left.PropertyName == "ID"
                    select (string)right.Value).FirstOrDefault();
        }

        internal static IEnumerable<ExpandoStatement> GetExpandoCalls(CodeMemberMethod buildMethod) {
            // Return all of the ((IAttributeAccessor)accessor).SetAttribute("", value) calls
            return from s in buildMethod.Statements.OfType<CodeExpressionStatement>()
                   let methodCall = s.Expression as CodeMethodInvokeExpression
                   // Get attribute accessor method calls
                   where methodCall != null && methodCall.Method.MethodName == "SetAttribute"
                   select new ExpandoStatement {
                       MethodCall = methodCall,
                       Statement = s
                   };
        }

        internal class ExpandoStatement {
            public CodeMethodInvokeExpression MethodCall { get; set; }
            // The owner of the method expression
            public CodeExpressionStatement Statement { get; set; }
        }
    }
}